/*************************************************************************
 CCOM  -  description
 -------------------
 début                : Décembre 2009
 copyright            : (C) 2009 par ESCANDELL Stéphane
 e-mail               : stephane.escandell@gmail.com
 *************************************************************************/

//---------- Réalisation de la classe <CCom> (fichier CCom.cpp) --

//---------------------------------------------------------------- INCLUDE

//-------------------------------------------------------- Include système
using namespace std;
//#define MAP 1
#include <iostream>
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>

//------------------------------------------------------ Include personnel
#include "CCom.h"

//------------------------------------------------------------- Constantes
/* Taille minimale d'un message envoyé par le Khepera avec les caratères
 * de fin de message
 */
static const unsigned int MIN_MSG_SIZE = 3; // 1 caractère + 2 caractères fin de message

/* Caractères de fin de chaîne envoyé par le Khepera pour identifier la fin
 * d'un message.
 */
static const char END_RCV_COMMAND_1 = 13;
static const char END_RCV_COMMAND_2 = 10;
//----------------------------------------------------------------- PUBLIC

//----------------------------------------------------- Méthodes publiques
void CCom::Close()
// Algorithme :
//
{
#ifdef MAP
	cout << "Fermeture du port COM" << endl;
#endif
	if (portOpened)
		close(fileDescriptor);
	portOpened = false;
} //--- Fin de <Close>


bool CCom::Open(string port)
// ALgorithme :
//	Trivial
//
{
#ifdef MAP
	cout << "Ouverture du port COM : " << port << endl;
#endif
	if (portOpened) {
		errno = ER_PORT_COM_ALREADY_OPENED;
		return false;
	}

	/* Ouverture du port COM <port> */
	if ((fileDescriptor = open(port.c_str(), O_RDWR | O_NOCTTY | O_NONBLOCK))
			< 0) {
		errno = ER_PORT_COM_NOT_OPENED;
		return false;
	}

	portOpened = true;
	return true;
} // --- Fin de <Open>


bool CCom::SetPortConfiguration(BaudRate baudRate, DataBits countBits,
		Parity parity, StopBits countBitStop)
// Algorithme :
//
{
	if (!portOpened) {
		errno = ER_PORT_COM_NOT_OPENED;
		return false;
	} else {
		/* Initialisation du port COM */
		// Modes locaux
		configuration.c_lflag &= ~ECHO;
		// Modes d'entrée
		configuration.c_iflag = IGNBRK | IGNPAR;
		// Modes de sortie
		configuration.c_oflag = 0;
		// Modes de contrôle
		configuration.c_cflag = CREAD | CLOCAL;
		configuration.c_cflag = configuration.c_cflag & ~CRTSCTS;
		// Vitesse
		switch (baudRate) {
		case CCom::CCOM_BR600:
			configuration.c_cflag |= B600;
			break;
		case CCom::CCOM_BR1200:
			configuration.c_cflag |= B1200;
			break;
		case CCom::CCOM_BR2400:
			configuration.c_cflag |= B2400;
			break;
		case CCom::CCOM_BR4800:
			configuration.c_cflag |= B4800;
			break;
		case CCom::CCOM_BR9600:
			configuration.c_cflag |= B9600;
			break;
		case CCom::CCOM_BR19200:
			configuration.c_cflag |= B19200;
			break;
		default:
			//Close();
			errno = CCom::ER_BAUD_RATE_UNDEFINED;
			return false;
		}
		// Longueur des caractères
		switch (countBits) {
		case CCom::CCOM_5_BITS:
			configuration.c_cflag |= CS5;
			break;
		case CCom::CCOM_6_BITS:
			configuration.c_cflag |= CS6;
			break;
		case CCom::CCOM_7_BITS:
			configuration.c_cflag |= CS7;
			break;
		case CCom::CCOM_8_BITS:
			configuration.c_cflag |= CS8;
			break;
		default:
			//Close();
			errno = CCom::ER_CAR_SIZE_UNDEFINED;
			return false;
		}
		// Parité
		switch (parity) {
		case CCom::CCOM_NONE: /* rien à faire */
			break;
		case CCom::CCOM_ODD:
			configuration.c_cflag |= PARODD;
		case CCom::CCOM_EVEN:
			configuration.c_cflag |= PARENB;
			break;
		default:
			//Close();
			errno = CCom::ER_PARITY_MODE_UNDEFINED;
			return false;
		}
		// Nombre de bits de stop
		switch (countBitStop) {
		case CCom::CCOM_1_BIT: /* rien à faire */
			break;
		case CCom::CCOM_2_BITS:
			configuration.c_cflag |= CSTOPB;
			break;
		default:
			//Close();
			errno = CCom::ER_COUNT_STOP_BIT_UNDEFINED;
			return false;
		}

		// Envoi de la configuration
		if (tcsetattr(fileDescriptor, TCSANOW, &configuration) == -1) {
			//Close();
			errno = CCom::ER_CONFIGURATION_UNAPPLICABLE;
			return false;
		}
		tcflush(fileDescriptor, TCIOFLUSH);
		return true;
	}
} // --- Fin de <SetPortConfiguration>


int CCom::ReceiveData(string & data)
// ALgorithme :
//	Répéter
//		Lire caractère
//		Affecter caractère lu à chaîne de destination
//	Répéter jusqu'à au moins avoir lu 3 caractères et que les 2 derniers
//	correspondent aux marqueurs de fin de message.
//
{
	char c_car;
	data.clear();
	do {
		//		cout << "data.length=" << data.length() << " MIN_MSG_SIZE=" << MIN_MSG_SIZE << endl;
		//		if (data.length() >= MIN_MSG_SIZE)
		//		{
		//			cout << "data.[data.length-2]=" << (data[data.length()-2]) << endl;
		//			cout << "data.[data.length-1]=" << (data[data.length()-1]) << endl;
		//		}
		if (read(fileDescriptor, &c_car, 1) > 0)
			//		{
			//			cout << "Car recu" << endl;
			data += c_car;
		//		}
		else
			usleep(500);
	}
	// Pour faire bien, il faudrait sortir la condition de retour et
	// la rendre indépendante du Khepera ... mais là on a pas le temps
	while (data.length() < MIN_MSG_SIZE || (data[data.length() - 2]
			!= END_RCV_COMMAND_1 && data[data.length() - 1]
			!= END_RCV_COMMAND_2));

	data.resize(data.length() - 2);
	return data.length();
} // --- Fin de <ReceiveData>

int CCom::SendData(string & data)
// Algorithme :
//	Trivial
//
{
	return write(fileDescriptor, data.c_str(), data.length());
} // --- Fin de <SendData>

string CCom::SetSpeed(int leftMotor, int rightMotor)
// Algorithme :
//	Trivial
{
	string return_data;
	char c_data[KH_SIZE_CMD_SPEED];
	sprintf(c_data, "%c%c%i%c%i%c", KH_CMD_SPEED, KH_CMD_SEPARATOR, leftMotor,
			KH_CMD_SEPARATOR, rightMotor, END_SEND_COMMAND);
	string data(c_data);
#ifdef MAP
	cout << "SetSpeed : " << data << endl;
#endif
	this->SendData(data);
	ReceiveData(return_data);

	return return_data;
}

string CCom::GetSpeed()
// Algorithme :
//	Trivial
{
	string return_data;
	char c_data[1];
	sprintf(c_data, "%c%c", 'E', END_SEND_COMMAND);
	string data(c_data);
#ifdef MAP
	cout << "GetSpeed : " << data << endl;
#endif
	this->SendData(data);
	ReceiveData(return_data);

	return return_data;
}

string CCom::ConfigurePIDController(int Kp, int Ki, int Kd)
// Algorithme :
//	Trivial
{
	string return_data;
	char c_data[KH_SIZE_CMD_PID];
	sprintf(c_data, "%c%c%i%c%i%c%i%c", KH_CMD_PID, KH_CMD_SEPARATOR, Kp,
			KH_CMD_SEPARATOR, Ki, KH_CMD_SEPARATOR, Kd, END_SEND_COMMAND);
	string data(c_data);
#ifdef MAP
	cout << "ConfigurePIDController : " << data << endl;
#endif
	this->SendData(data);
	ReceiveData(return_data);

	return return_data;
}

string CCom::ConfigureSpeedProfileController(int maxSpeedLeft, int accLeft,
		int maxSpeedRight, int accRight)
// Algorithme :
//	Trivial
{
	string return_data;
	char c_data[KH_SIZE_CMD_SPEED_PROFILE];
	sprintf(c_data, "%c%c%i%c%i%c%i%c%i%c", KH_CMD_SPEED_PROFILE,
			KH_CMD_SEPARATOR, maxSpeedLeft, KH_CMD_SEPARATOR, accLeft,
			KH_CMD_SEPARATOR, maxSpeedRight, KH_CMD_SEPARATOR, accRight,
			END_SEND_COMMAND);
	string data(c_data);
#ifdef MAP
	cout << "ConfigureSpeedProfileController : " << data << endl;
#endif
	this->SendData(data);
	ReceiveData(return_data);

	return return_data;
}

string CCom::ChangeLEDState(int number, int action)
// Algorithme :
//	Trivial
{
	string return_data;
	char c_data[KH_SIZE_CMD_LED];
	sprintf(c_data, "%c%c%i%c%i%c", KH_CMD_LED, KH_CMD_SEPARATOR, number,
			KH_CMD_SEPARATOR, action, END_SEND_COMMAND);
	string data(c_data);
#ifdef MAP
	cout << "ChangeLEDState : " << data << endl;
#endif
	this->SendData(data);
	ReceiveData(return_data);

	return return_data;
}

//------------------------------------------------- Surcharge d'opérateurs
//CCom & CCom::operator = ( const CCom & aCCom )
// Algorithme :
//
//{
//} //----- Fin de operator =


//-------------------------------------------- Constructeurs - destructeur
/*CCom::CCom ( const CCom & aCCom )
 // Algorithme :
 //
 {
 #ifdef MAP
 cout << "Appel au constructeur de copie de <CCom>" << endl;
 #endif
 } //----- Fin de CCom (constructeur de copie)
 */

CCom::CCom()
// Algorithme :
//
{
#ifdef MAP
	cout << "Appel au constructeur de <CCom>" << endl;
#endif
	portOpened = false;
} //----- Fin de CCom

CCom::~CCom()
// Algorithme :
//
{
#ifdef MAP
	cout << "Appel au destructeur de <CCom>" << endl;
#endif
	if (portOpened)
		close(fileDescriptor);
}
//------------------------------------------------------------------ PRIVE

//----------------------------------------------------- Méthodes protégées
