/*************************************************************************
                           CCom  -  description
                             -------------------
    début                : $DATE$
    copyright            : (C) $YEAR$ par $AUTHOR$
    e-mail               : $EMAIL$
*************************************************************************/

//---------- Interface de la classe <CCom> (fichier CCom.h) ------
#if ! defined ( CCOM_H )
#define CCOM_H

//--------------------------------------------------- Interfaces utilisées
using namespace std;
#include <string>
#include <termios.h>

//------------------------------------------------------------- Constantes
// Constantes générales
static const char END_SEND_COMMAND = '\n';
static const char KH_CMD_SEPARATOR = ',';

// Constante Set_Speed
static const char KH_CMD_SPEED = 'D';
const char KH_RETURN_CMD_SPEED = 'd';
static const int KH_SIZE_CMD_SPEED = 5;

// Constante Configure Speed Profile
static const char KH_CMD_SPEED_PROFILE = 'J';
const char KH_RETURN_CMD_SPEED_PROFILE = 'j';
static const int KH_SIZE_CMD_SPEED_PROFILE = 5;

// Constante Configure PID
static const char KH_CMD_PID = 'F';
const char KH_RETURN_CMD_PID = 'f';
static const int KH_SIZE_CMD_PID = 7;

// Constante Change LED state
static const char KH_CMD_LED = 'L';
const char KH_RETURN_CMD_LED = 'l';
static const int KH_SIZE_CMD_LED = 9;


//------------------------------------------------------------------ Types

//------------------------------------------------------------------------
// Rôle de la classe <CCom>
//
//
//------------------------------------------------------------------------

class CCom
{
//----------------------------------------------------------------- PUBLIC

public:
//----------------------------------------------------------- Enumérations
	enum BaudRate
	{
		CCOM_BR19200,
		CCOM_BR9600,
		CCOM_BR4800,
		CCOM_BR2400,
		CCOM_BR1200,
		CCOM_BR600
	};
	enum Erreur
	{
		ER_CCOM = -1000,
		ER_PORT_COM_NOT_OPENED 			= CCom::ER_CCOM-1,
		ER_PORT_COM_ALREADY_OPENED		= CCom::ER_CCOM-2,
		ER_BAUD_RATE_UNDEFINED 			= CCom::ER_CCOM-3,
		ER_CAR_SIZE_UNDEFINED  			= CCom::ER_CCOM-4,
		ER_PARITY_MODE_UNDEFINED  		= CCom::ER_CCOM-5,
		ER_COUNT_STOP_BIT_UNDEFINED 	= CCom::ER_CCOM-6,
		ER_CONFIGURATION_UNAPPLICABLE 	= CCom::ER_CCOM-7
	};
	enum Parity
	{
		CCOM_NONE,
		CCOM_EVEN, /* pair */
		CCOM_ODD
	};
	enum DataBits
	{
		CCOM_8_BITS,
		CCOM_7_BITS,
		CCOM_6_BITS,
		CCOM_5_BITS
	};
	enum StopBits
	{
		CCOM_1_BIT,
		CCOM_2_BITS
	};

//----------------------------------------------------- Méthodes publiques
    // type Méthode ( liste des paramètres );
    // Mode d'emploi :
    //
    // Contrat :
    //
	void Close ();
	// Mode d'emploi :
	//
	// Contrat :
	//

	bool Open ( string port );
	// Mode d'emploi :
	//
	// Contrat :
	//

	int ReceiveData(string & data);
	// Mode d'emploi :
	//		Lit sur le port COM jusqu'à avoir au moins MIN_MSG_SIZE (3)
	//	caractères et avoir lu la suite de caractères "\r\c".
	//		Le contenu de <data> est automatiquement réinitialisé.
	//		<data> ne contiendra pas les marqueurs de fin de message.
	//		Renvoie le nombre de caractères lus sauf les marqueurs de fin de
	// 	message.
	//
	// Contrat :
	//	Le port COM a dû préalablement être ouvert par appel à la méthode
	//	<Open>
	//

	int SendData(string & data);
	// Mode d'emploi :
	//	Envoie le message <data> sur le port de communication préalablement initialisé.
	//	Renvoie le nombre de caractères écrits.
	//
	// Contrat :
	//  <data> ne doit pas contenir le doublet de fin de message.
	//	La méthode <Open> a dû être appelée auparavant avec succès.
	//

	bool SetPortConfiguration(BaudRate baudRate, DataBits countBits, Parity parity, StopBits countBitsStop);
	// Mode d'emploi :
	//
	// Contrat :
	//

	string SetSpeed(int leftMotor, int rightMotor);
	// Mode d'emploi :
	//
	// Contrat :
	//

	string GetSpeed();
	// Mode d'emploi :
	//
	// Contrat :
	//

	string ConfigurePIDController(int Kp, int Ki, int Kd);
	// Mode d'emploi :
	//
	// Contrat :
	//

	string ConfigureSpeedProfileController(int maxSpeedLeft, int accLeft, int maxSpeedRight, int accRight);
	// Mode d'emploi :
	//
	// Contrat :
	//

	string ChangeLEDState(int number, int action);
	// Mode d'emploi :
	//
	// Contrat :
	//

//------------------------------------------------- Surcharge d'opérateurs
    //CCom & operator = ( const CCom & aCCom );
    // Mode d'emploi :
    //
    // Contrat :
    //


//-------------------------------------------- Constructeurs - destructeur
    //CCom ( const CCom & aCCom );
    // Mode d'emploi (constructeur de copie) :
    //
    // Contrat :
    //

    CCom ( );
    // Mode d'emploi :
    //
    // Contrat :
    //

    virtual ~CCom ( );
    // Mode d'emploi :
    //
    // Contrat :
    //

//------------------------------------------------------------------ PRIVE

protected:
//----------------------------------------------------- Méthodes protégées
    // void handler(int signal);
//----------------------------------------------------- Attributs protégés
    int fileDescriptor;
    bool portOpened;
    termios configuration;
};

//--------------------------- Autres définitions dépendantes de <CCom>

#endif // CCOM_H
